function cpm_main

% Constant panel method for a 2D Neumann Problem. April 12, 2006.
% Compute velocity potential, velocities, and pressure on body surface
% by using the source method
% Example: uniform flow past an arbitrary body in an infinite fluid.
% The program is written for arbitrary body geometry and flow speed.
% Results are plotted for a circular cylinder of radius R=2m and
% uniform flow speed U=1m/s.
close all
% input flow speed and fluid density
% u = input(' Enter Flow Speed: ');
% rho = 1000; % water density m/s.
alpha=0;%pi/8; %angle of attack
u=-1*[cos(alpha) sin(alpha)];rho=1;  %They assume -Ux+phi(....)

% input body geometry and obtain basic panel information 
[npanels,xvert,yvert,xc,yc,len,nx,ny] = setpanels;

% evaluate the polar angle of the centroid of each panel
%(angle varies from 0 to 2pi)
thc = atan2(yc,xc);
for k=1:npanels;
    if thc(k) < 0;
        thc(k) = thc(k)+2*pi;
    end;
end;

%% build up the system of equation: Ax=b with x being unknown source
%% strength, A and b being influece coefficients

% find G(x,y;xi,zeta),Gx(x,y;xi,zeta), Gy(x,y;xi,zeta), i.e. potential G,
% x-velocity Gx and y-velocity at centroid of panel (x,y) due to constant
% source distribuiton on panel (xi,zeta).
[g,gx,gy] =inflcoef(npanels,xvert,yvert,xc,yc,nx,ny,len);

% find normal velocity on body surface
dpotdn = setbcs(u(1),nx)+u(2)*ny;

% determine influence coefficients A and b
[A,b] = matrices(npanels,nx,ny,gx,gy,dpotdn);

% solve Ax=b for unknown source strength
source_strength = A\b';

%% Post Processing by Reza Karimi
step=.01;% Different steps for contour or quiver plots
x_set=[-1:step:1.2]; %change x_min  to -1 for streamlines
y_set=[-.4:step:.4]; 
ONES=ones(size(y_set));
PHI=zeros(length(y_set),length(x_set));
Vx=PHI;
Vy=PHI;
i=0;
for xi=x_set
    i=i+1;
    [g,gx,gy] =inflcoef(npanels,xvert,yvert,xi*ONES,y_set,nx,ny,len);
    PHI(:,i)=g*source_strength;
    Vx(:,i)=gx*source_strength-u(1);
    Vy(:,i)=gy*source_strength-u(2);
end
V=(Vx.^2+Vy.^2).^0.5/norm(u);
ELIMNATE_POINTS=find(V==inf);
V(ELIMNATE_POINTS)=NaN;
Vx(ELIMNATE_POINTS)=NaN;
Vy(ELIMNATE_POINTS)=NaN;


P=1/2*rho*(norm(u)^2-norm(u)^2*V.^2)+0; % Note that V is already normalized

% At the center
[g,gx,gy] =inflcoef(npanels,xvert,yvert,xc,yc,nx,ny,len);
Vcx=gx*source_strength-u(1);
Vcy=gy*source_strength-u(2);
Vc=(Vcx.^2+Vcy.^2).^0.5/norm(u);
V_max=max(max(Vc), max(max(V)))

%% Figures
figure,contourf(x_set,y_set,V,20),colorbar
%surfc(x_set,y_set,0*V,V),view([0 0 -1])
hold on,Fig_Set_Basic('Velocity Contours $\frac{\|\vec{V}\|}{U_{\infty}}$'), %shading interp

figure,contourf(x_set,y_set,P/(rho/norm(u)^2),20),colorbar
hold on,Fig_Set_Basic('Pressure Contours: $\frac{P-P_{\infty}}{\rho U_{\infty}^2}$'), %shading interp

figure,quiver(x_set,y_set,Vx,Vy)%,step*1.5)
hold on,Fig_Set_Basic('Velocity Vectors: $\vec{V}$')

figure, quiver(xc,yc,Vcx',Vcy')%,step*1.5)
hold on,Fig_Set_Basic('Velocity at Source Centers $\vec{V_c}$')
% 
S_y=[min(y_set):.025:max(y_set)]; %When alpha is zero
figure, streamline(x_set,y_set,Vx,Vy,min(x_set)*ones(size(S_y)),S_y),hold on
hold on,Fig_Set_Basic('Streamlines')

figure,%When alpha is not zero
x_base=min(x_set)/2;
stream_y= S_y*cos(alpha)+x_base*sin(alpha);
stream_x=-S_y*sin(alpha)+x_base*cos(alpha);
streamline(x_set,y_set,Vx,Vy,stream_x,stream_y),hold on
streamline(x_set,y_set,-Vx,-Vy,stream_x,stream_y),hold on
axis([-1 1.2 -4. .4])
% % x_base=max(x_set)/2;
% % stream_y= S_y*cos(alpha)+x_base*sin(alpha);
% % stream_x=-S_y*sin(alpha)+x_base*cos(alpha);
% % streamline(x_set,y_set,-Vx,-Vy,stream_x,stream_y),hold on
% % streamline(x_set,y_set, Vx, Vy,stream_x,stream_y),hold on
hold on,Fig_Set_Basic('Streamlines')

%%%%%%%%%%%%%%%%%%%%%
    function Fig_Set_Basic(Title)
        fill(xvert,yvert,'y')
        XTITLE='$\frac{x}{c}$  (c: Chord Length)';YTITLE='$\frac{y}{c}$';
        xlabel(XTITLE,'Interpreter','latex','fontsize',20,'FontAngle','italic','FontWeight','bold','FontName','Times New Roman')
        ylabel(YTITLE,'Interpreter','latex','fontsize',20,'FontAngle','italic','FontWeight','bold','FontName','Times New Roman')
        title (Title,'Interpreter','latex','fontsize',16,'FontAngle','italic','FontWeight','bold','FontName','Times New Roman')
        set(gcf,'Position',[50 50 [25 15]*25],'color','w'),set(gca,'fontsize',12)
        box on, grid on,axis equal,
        axis([min(x_set) max(x_set) min(y_set) max(y_set)])
    end
end

%% Original Code for Cylinder
% % % %% evaluate velocity potential on body surface:
% % % % for i=1:npanels
% % % %     phi(i)=g(i,:)*source_strength;
% % % % end
% % % 
% % % % compare the numerical solution to the analytic solution for the
% % % % potential on body surface (for a circular cylinder)
% % % % figure(1)
% % % % %phi_analytic=-u*xc;
% % % % plot(thc,phi,'r-',thc,phi_analytic,'g*')
% % % % xlabel('angle');
% % % % ylabel('velocity potential');
% % % % legend('numerical result','analytical result');
% % % %
% % % %
% % % % evaluate tangential velocity on the body surface
% % % % and compare with the analytical solution
% % % % for i=1:npanels
% % % % numerical solution
% % % %     vtc(i)=(gx(i,:)*source_strength)*ny(i)+ ...
% % % %            (gy(i,:)*source_strength)*(-nx(i))-u*ny(i);
% % % % analytic solution
% % % %       vth(i) = 2.0*u*sin(thc(i));
% % % % end
% % % %
% % % % figure(2)
% % % % plot(thc, vtc, 'r-')%, thc, vth, 'g*');
% % % % xlabel('angle');
% % % % ylabel('tangential velocity');
% % % % legend('numerical result','analytical result');
% % % %
% % % % evaluate the distribution of the presure coefficient on the body
% % % % p = -0.5*rho*(vtc .* vtc - u*u);
% % % % cp = p/(0.5*rho*u*u);
% % % % cp_analytical=1-4*sin(thc).^2;
% % % %
% % % % figure(3)
% % % % plot(thc,cp,'r-',thc,cp_analytical,'g*');
% % % % xlabel('angle');
% % % % ylabel ('Pressure Coefficient (Cp)');
% % % % legend('numerical result','analytical result');